/** Management class and utils for Security S2 */
/// <reference types="node" />
import { CtrDRBG } from "./ctr_drbg";
import { SecurityClass } from "./SecurityClass";
interface NetworkKeys {
    pnk: Buffer;
    keyCCM: Buffer;
    keyMPAN: Buffer;
    personalizationString: Buffer;
}
interface TempNetworkKeys {
    keyCCM: Buffer;
    personalizationString: Buffer;
}
export declare enum SPANState {
    /** No entry exists */
    None = 0,
    RemoteEI = 1,
    LocalEI = 2,
    SPAN = 3
}
export type SPANTableEntry = {
    type: SPANState.RemoteEI;
    receiverEI: Buffer;
} | {
    type: SPANState.LocalEI;
    receiverEI: Buffer;
} | {
    type: SPANState.SPAN;
    securityClass: SecurityClass;
    rng: CtrDRBG;
    /** The most recent generated SPAN */
    currentSPAN?: {
        nonce: Buffer;
        expires: number;
    };
};
export declare class SecurityManager2 {
    constructor();
    /** PRNG used to initialize the others */
    private rng;
    /** A map of SPAN states for each node */
    private spanTable;
    /** A map of temporary keys for each node that are used for the key exchange */
    readonly tempKeys: Map<number, TempNetworkKeys>;
    /** A map of sequence numbers that were last used in communication with a node */
    private ownSequenceNumbers;
    private peerSequenceNumbers;
    /** A map of MPAN states for each multicast group */
    private mpanStates;
    /** A map of permanent network keys per security class */
    private networkKeys;
    /** Which multicast group has been assigned which security class */
    private groupClasses;
    /** Sets the PNK for a given security class and derives the encryption keys from it */
    setKey(securityClass: SecurityClass, key: Buffer): void;
    assignSecurityClassMulticast(group: number, securityClass: SecurityClass): void;
    hasKeysForSecurityClass(securityClass: SecurityClass): boolean;
    getKeysForSecurityClass(securityClass: SecurityClass): NetworkKeys;
    getKeysForNode(peerNodeID: number): NetworkKeys | TempNetworkKeys;
    getSPANState(peerNodeID: number): SPANTableEntry | {
        type: SPANState.None;
    };
    /** Tests whether the most recent secure command for a node has used the given security class */
    hasUsedSecurityClass(peerNodeID: number, securityClass: SecurityClass): boolean;
    /**
     * Prepares the generation of a new SPAN by creating a random sequence number and (local) entropy input
     * @param receiver The node this nonce is for. If none is given, the nonce is not stored.
     */
    generateNonce(receiver: number | undefined): Buffer;
    /**
     * Stores the given SPAN state in the table. This should NEVER be called by user code.
     */
    setSPANState(peerNodeID: number, state: SPANTableEntry | {
        type: SPANState.None;
    }): void;
    /** Invalidates the SPAN state for the given receiver */
    deleteNonce(receiver: number): void;
    /** Initializes the singlecast PAN generator for a given node based on the given entropy inputs */
    initializeSPAN(peerNodeId: number, securityClass: SecurityClass, senderEI: Buffer, receiverEI: Buffer): void;
    /** Initializes the singlecast PAN generator for a given node based on the given entropy inputs */
    initializeTempSPAN(peerNodeId: number, senderEI: Buffer, receiverEI: Buffer): void;
    /** Tests if the given combination of peer node ID and sequence number is a duplicate */
    isDuplicateSinglecast(peerNodeId: number, sequenceNumber: number): boolean;
    /** Stores the latest sequence number for the given peer node ID and returns the previous one */
    storeSequenceNumber(peerNodeId: number, sequenceNumber: number): number | undefined;
    storeRemoteEI(peerNodeId: number, remoteEI: Buffer): void;
    /**
     * Generates the next nonce for the given peer and returns it.
     * @param store - Whether the nonce should be stored/remembered as the current SPAN.
     */
    nextNonce(peerNodeId: number, store?: boolean): Buffer;
    /** Returns the next sequence number to use for outgoing messages to the given node */
    nextSequenceNumber(peerNodeId: number): number;
    initializeMPAN(group: number): void;
    nextMPAN(group: number): Buffer;
}
export {};
//# sourceMappingURL=Manager2.d.ts.map