"use strict";
/** Management class and utils for Security S2 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SecurityManager2 = exports.SPANState = void 0;
const shared_1 = require("@zwave-js/shared");
const crypto = __importStar(require("crypto"));
const ZWaveError_1 = require("../error/ZWaveError");
const date_1 = require("../util/date");
const bufferUtils_1 = require("./bufferUtils");
const crypto_1 = require("./crypto");
const ctr_drbg_1 = require("./ctr_drbg");
const SecurityClass_1 = require("./SecurityClass");
var SPANState;
(function (SPANState) {
    /** No entry exists */
    SPANState[SPANState["None"] = 0] = "None";
    /* The other node's receiver's entropy input is known but, but we didn't send it our sender's EI yet */
    SPANState[SPANState["RemoteEI"] = 1] = "RemoteEI";
    /* We've sent the other node our receiver's entropy input, but we didn't receive its sender's EI yet */
    SPANState[SPANState["LocalEI"] = 2] = "LocalEI";
    /* An SPAN with the other node has been established */
    SPANState[SPANState["SPAN"] = 3] = "SPAN";
})(SPANState = exports.SPANState || (exports.SPANState = {}));
// How many sequence numbers are remembered for each node when checking for duplicates
const SINGLECAST_MAX_SEQ_NUMS = 10;
// How long a singlecast nonce used for encryption will be kept around to attempt decryption of in-flight messages
const SINGLECAST_NONCE_EXPIRY_NS = 500 * 1000 * 1000; // 500 ms in nanoseconds
class SecurityManager2 {
    constructor() {
        /** A map of SPAN states for each node */
        this.spanTable = new Map();
        /** A map of temporary keys for each node that are used for the key exchange */
        this.tempKeys = new Map();
        /** A map of sequence numbers that were last used in communication with a node */
        this.ownSequenceNumbers = new Map();
        this.peerSequenceNumbers = new Map();
        /** A map of MPAN states for each multicast group */
        this.mpanStates = new Map();
        /** A map of permanent network keys per security class */
        this.networkKeys = new Map();
        /** Which multicast group has been assigned which security class */
        this.groupClasses = new Map();
        this.rng = new ctr_drbg_1.CtrDRBG(128, false, crypto.randomBytes(32), undefined, Buffer.alloc(32, 0));
    }
    /** Sets the PNK for a given security class and derives the encryption keys from it */
    setKey(securityClass, key) {
        if (key.length !== 16) {
            throw new ZWaveError_1.ZWaveError(`The network key must consist of 16 bytes!`, ZWaveError_1.ZWaveErrorCodes.Argument_Invalid);
        }
        else if (!(securityClass in SecurityClass_1.SecurityClass) ||
            securityClass <= SecurityClass_1.SecurityClass.None) {
            throw new ZWaveError_1.ZWaveError(`Invalid security class!`, ZWaveError_1.ZWaveErrorCodes.Argument_Invalid);
        }
        this.networkKeys.set(securityClass, {
            pnk: key,
            ...(0, crypto_1.deriveNetworkKeys)(key),
        });
    }
    assignSecurityClassMulticast(group, securityClass) {
        this.groupClasses.set(group, securityClass);
    }
    hasKeysForSecurityClass(securityClass) {
        return this.networkKeys.has(securityClass);
    }
    getKeysForSecurityClass(securityClass) {
        const keys = this.networkKeys.get(securityClass);
        if (!keys) {
            throw new ZWaveError_1.ZWaveError(`The network key for the security class ${(0, shared_1.getEnumMemberName)(SecurityClass_1.SecurityClass, securityClass)} has not been set up yet!`, ZWaveError_1.ZWaveErrorCodes.Security2CC_NotInitialized);
        }
        return { ...keys };
    }
    getKeysForNode(peerNodeID) {
        const spanState = this.getSPANState(peerNodeID);
        // The keys we return must match the actual SPAN state (if we have one)
        // Meaning if an SPAN for the temporary inclusion key is established,
        // we need to return that temporary key
        if (spanState.type === SPANState.SPAN &&
            spanState.securityClass === SecurityClass_1.SecurityClass.Temporary) {
            if (this.tempKeys.has(peerNodeID))
                return this.tempKeys.get(peerNodeID);
            throw new ZWaveError_1.ZWaveError(`Temporary encryption key for node ${peerNodeID} is not known!`, ZWaveError_1.ZWaveErrorCodes.Security2CC_NotInitialized);
        }
        else if (spanState.type !== SPANState.SPAN) {
            throw new ZWaveError_1.ZWaveError(`Security class for node ${peerNodeID} is not yet known!`, ZWaveError_1.ZWaveErrorCodes.Security2CC_NotInitialized);
        }
        return this.getKeysForSecurityClass(spanState.securityClass);
    }
    getSPANState(peerNodeID) {
        return this.spanTable.get(peerNodeID) ?? { type: SPANState.None };
    }
    /** Tests whether the most recent secure command for a node has used the given security class */
    hasUsedSecurityClass(peerNodeID, securityClass) {
        const spanState = this.spanTable.get(peerNodeID);
        if (!spanState)
            return false;
        if (spanState.type !== SPANState.SPAN)
            return false;
        return spanState.securityClass === securityClass;
    }
    /**
     * Prepares the generation of a new SPAN by creating a random sequence number and (local) entropy input
     * @param receiver The node this nonce is for. If none is given, the nonce is not stored.
     */
    generateNonce(receiver) {
        const receiverEI = this.rng.generate(16);
        if (receiver != undefined) {
            this.spanTable.set(receiver, {
                type: SPANState.LocalEI,
                receiverEI,
            });
        }
        return receiverEI;
    }
    /**
     * Stores the given SPAN state in the table. This should NEVER be called by user code.
     */
    setSPANState(peerNodeID, state) {
        if (state.type === SPANState.None) {
            this.spanTable.delete(peerNodeID);
        }
        else {
            this.spanTable.set(peerNodeID, state);
        }
    }
    /** Invalidates the SPAN state for the given receiver */
    deleteNonce(receiver) {
        this.spanTable.delete(receiver);
        this.peerSequenceNumbers.delete(receiver);
        // Keep our own sequence number
    }
    /** Initializes the singlecast PAN generator for a given node based on the given entropy inputs */
    initializeSPAN(peerNodeId, securityClass, senderEI, receiverEI) {
        if (senderEI.length !== 16 || receiverEI.length !== 16) {
            throw new ZWaveError_1.ZWaveError(`The entropy input must consist of 16 bytes`, ZWaveError_1.ZWaveErrorCodes.Argument_Invalid);
        }
        const keys = this.getKeysForSecurityClass(securityClass);
        const noncePRK = (0, crypto_1.computeNoncePRK)(senderEI, receiverEI);
        const MEI = (0, crypto_1.deriveMEI)(noncePRK);
        this.spanTable.set(peerNodeId, {
            securityClass,
            type: SPANState.SPAN,
            rng: new ctr_drbg_1.CtrDRBG(128, false, MEI, undefined, keys.personalizationString),
        });
    }
    /** Initializes the singlecast PAN generator for a given node based on the given entropy inputs */
    initializeTempSPAN(peerNodeId, senderEI, receiverEI) {
        if (senderEI.length !== 16 || receiverEI.length !== 16) {
            throw new ZWaveError_1.ZWaveError(`The entropy input must consist of 16 bytes`, ZWaveError_1.ZWaveErrorCodes.Argument_Invalid);
        }
        const keys = this.tempKeys.get(peerNodeId);
        const noncePRK = (0, crypto_1.computeNoncePRK)(senderEI, receiverEI);
        const MEI = (0, crypto_1.deriveMEI)(noncePRK);
        this.spanTable.set(peerNodeId, {
            securityClass: SecurityClass_1.SecurityClass.Temporary,
            type: SPANState.SPAN,
            rng: new ctr_drbg_1.CtrDRBG(128, false, MEI, undefined, keys.personalizationString),
        });
    }
    /** Tests if the given combination of peer node ID and sequence number is a duplicate */
    isDuplicateSinglecast(peerNodeId, sequenceNumber) {
        return (this.peerSequenceNumbers
            .get(peerNodeId)
            ?.includes(sequenceNumber) ?? false);
    }
    /** Stores the latest sequence number for the given peer node ID and returns the previous one */
    storeSequenceNumber(peerNodeId, sequenceNumber) {
        if (this.peerSequenceNumbers.has(peerNodeId)) {
            // Store the last SINGLECAST_MAX_SEQ_NUMS sequence numbers
            const arr = this.peerSequenceNumbers.get(peerNodeId);
            const prev = arr[arr.length - 1];
            arr.push(sequenceNumber);
            if (arr.length > SINGLECAST_MAX_SEQ_NUMS)
                arr.shift();
            return prev;
        }
        else {
            this.peerSequenceNumbers.set(peerNodeId, [sequenceNumber]);
        }
    }
    storeRemoteEI(peerNodeId, remoteEI) {
        if (remoteEI.length !== 16) {
            throw new ZWaveError_1.ZWaveError(`The entropy input must consist of 16 bytes`, ZWaveError_1.ZWaveErrorCodes.Argument_Invalid);
        }
        this.spanTable.set(peerNodeId, {
            type: SPANState.RemoteEI,
            receiverEI: remoteEI,
        });
    }
    /**
     * Generates the next nonce for the given peer and returns it.
     * @param store - Whether the nonce should be stored/remembered as the current SPAN.
     */
    nextNonce(peerNodeId, store) {
        const spanState = this.spanTable.get(peerNodeId);
        if (spanState?.type !== SPANState.SPAN) {
            throw new ZWaveError_1.ZWaveError(`The Singlecast PAN has not been initialized for Node ${peerNodeId}`, ZWaveError_1.ZWaveErrorCodes.Security2CC_NotInitialized);
        }
        const nonce = spanState.rng.generate(16).slice(0, 13);
        spanState.currentSPAN = store
            ? {
                nonce,
                expires: (0, date_1.highResTimestamp)() + SINGLECAST_NONCE_EXPIRY_NS,
            }
            : undefined;
        return nonce;
    }
    /** Returns the next sequence number to use for outgoing messages to the given node */
    nextSequenceNumber(peerNodeId) {
        let seq = this.ownSequenceNumbers.get(peerNodeId);
        if (seq == undefined) {
            seq = crypto.randomInt(256);
        }
        else {
            seq = (seq + 1) & 0xff;
        }
        this.ownSequenceNumbers.set(peerNodeId, seq);
        return seq;
    }
    initializeMPAN(group) {
        this.mpanStates.set(group, this.rng.generate(16));
    }
    nextMPAN(group) {
        if (!this.mpanStates.has(group)) {
            throw new ZWaveError_1.ZWaveError(`The MPAN state has not been initialized for multicast group ${group}`, ZWaveError_1.ZWaveErrorCodes.Security2CC_NotInitialized);
        }
        else if (!this.groupClasses.has(group)) {
            throw new ZWaveError_1.ZWaveError(`Multicast group ${group} has not been assigned to a security class yet!`, ZWaveError_1.ZWaveErrorCodes.Security2CC_NotInitialized);
        }
        const keys = this.networkKeys.get(this.groupClasses.get(group));
        if (!keys) {
            throw new ZWaveError_1.ZWaveError(`The network keys for the security class of multicast group ${group} have not been set up yet!`, ZWaveError_1.ZWaveErrorCodes.Security2CC_NotInitialized);
        }
        // Compute the next MPAN
        const stateN = this.mpanStates.get(group);
        const ret = (0, crypto_1.encryptAES128ECB)(stateN, keys.keyMPAN);
        // Increment the inner state
        (0, bufferUtils_1.increment)(stateN);
        return ret;
    }
}
exports.SecurityManager2 = SecurityManager2;
//# sourceMappingURL=Manager2.js.map